<?php
/**
 * Plugin Name: 丸子管理设置框架
 * Plugin URI: https://www.weitimes.com
 * Description: 丸子小程序团队基于 WordPress 创建管理页面设置框架
 * Version: 1.0.0
 * Author: 丸子团队
 * Author URI: https://www.imahui.com
 * requires at least: 4.9.8
 * tested up to: 6.1
 **/

class WanziFramework {

	private $option;  // 数据存储选项

	private $fields;  // 管理设置字段
    
    private $setting; // 存储数据集

	public function __construct( $fields = [], $option = 'minapp' ) {
        $this->option    = $option;
        $this->fields    = $fields;
        $this->setting   = get_option( $option );
        //add_action( 'admin_notices', array( $this, 'wanzi_framework_admin_notice' ) );
        add_action( 'admin_enqueue_scripts', function ( ) {
            if( function_exists( 'add_thickbox' ) ) {
                add_thickbox( );
            }
	    } );
    }
    
    public function wanzi_framework_admin_notice( ) {
        $screen = get_current_screen( );
    	if( isset($_GET['page']) && isset($_REQUEST['settings-updated']) ) {
    		if( $screen->id !== 'toplevel_page_'.trim($_GET['page']) ) return;
    		if( 'true' === $_GET['settings-updated'] ) {
    			$class = 'notice notice-success is-dismissible';
    			$message = __( '设置已更新保存!', 'imahui' );
    		} else {
    			$class = 'notice notice-warning is-dismissible';
    			$message = __( '对不起，更新出错啦，请检查!', 'imahui' );
    		}
    		printf( '<div class="%1$s"><p><strong>%2$s</strong></p></div>', esc_attr( $class ), esc_html( $message ) );
    	}
    }

	public function wanzi_option_framework_title( ) {
        $output  = '';
        $options = $this->fields;
        if( !empty($options) ) {
            if( count($options) === 1 ) {
                foreach ( $options as $key => $option ) {
                    $output .= '<h1>'.esc_html( $option['title'] ).'</h1>';
                }
            } else {
                $output     .= '<h2 class="wp-nav-tab-wrapper wp-clearfix">';
        		foreach ( $options as $key => $option ) {
        			$output .= '<a id="'.$key. '-tab" class="wp-nav-tab ' .$key.'-tab" title="' . esc_attr( $option['title'] ) . '" href="#'.$key.'">' . esc_html( $option['title'] ) . '</a>';
        		}
        		$output     .= '</h2>';
            }
        }
        printf( $output );
    }

	public function wanzi_option_framework_container( ) {
		do_action( "wanzi_team_activty_bulletin");
		$options = $this->fields;
		printf( '<div class="wrap">' );
		$this->wanzi_option_framework_title( );
		if( !empty($options) ) {
			printf( '<div id="section" class="section-container wp-clearfix">' );
			printf( '<form id="%s" method="post" action="options.php" enctype="multipart/form-data">', esc_attr( $this->option ) );
			settings_fields( $this->option.'-group' );
			foreach ( $options as $key => $option ) {
				printf( '<div id="%s" class="options-group">', esc_attr( $key ) );
                if( isset($option["summary"]) ) {
				    printf( '<h3>%s</h3>', $option["summary"] );
                }
				$this->wanzi_option_framework_sections( $option );
				printf(  '</div>' );
			}
            submit_button( );
			printf( '</form>' );
			printf( '</div>' );
		} else {
			printf( '<p>未定义设置选项</p>' );
		}
		printf( '</div><!-- / .wrap -->' );
		printf( '<div id="scroll-bar">' );
		printf( '<div id="goTop" class="scroll-up"><a href="javascript:;" title="直达顶部">▲</a></div>' );
		printf( '<div id="down" class="scroll-down"><a href="javascript:;" title="直达底部">▼</a></div>' );
		printf( '</div>' );
	}

	public function wanzi_option_framework_sections( $option ) {
		$fields = isset($option["fields"]) ? $option["fields"] : [];
		$setting = $this->setting;
		if( ! empty($fields) ) {
			printf( '<table class="form-table" cellspacing="0">' );
			printf( '</tbody>' );
			foreach ( $fields as $var => $field ) {
				$data 		= isset($setting[$var]) ? $setting[$var] : '';
				$default 	= isset($field["default"]) ? $field["default"] : "";
				$role    	= isset($field["role"]) ? current_user_can($field["role"]) : current_user_can('manage_options');
				$callback 	= 'wanzi_framework_'.str_replace("-", "_", $field['type']).'_callback';
				$params 	= array(
					"id" => $var,
					"option" => $this->option.'['.$var.']',
					"field" => $field,
					"default" => $default,
					"data" => $data
				);
				if( $role ) {
					printf( '<tr class="%s_%s">', esc_attr( $var ), esc_attr( $field['type'] ) );
					printf( '<th><label for="%s">%s</label></th>', esc_attr( $var ), esc_html( $field["title"] ) );
					printf( '<td>' );
					if( method_exists( $this, $callback ) ) {
						call_user_func_array( array($this, $callback), array( $params ) );
					} else {
						$this->wanzi_framework_default_callback( $params );
					}
					printf( '</td>' );
					printf( '</tr>' );
				}
			}
			printf( '</tbody>' );
			printf( '</table>' );
		}
	}

	public function wanzi_framework_default_callback( $args ) {
		if( has_action( 'wanzi_framework_default_callback' ) ) {
            do_action( 'wanzi_framework_default_callback', $args );
		} else {
        	$this->wanzi_text_callback( $args );
		}
	}

	public function wanzi_framework_text_callback( $args ) {
		$input          = $args['id'];
        $data        	= $args['data'];
        $fields         = $args['field'];
		$option 		= $args['option'];
        $default        = $args["default"];
        $value          = $data ? $data : $default;
        $class          = isset($fields["class"]) ? 'class="'.trim($fields["class"]).'"' : '';
        $descript       = isset($fields["description"]) ? trim($fields["description"]) : '';
        $placeholder    = isset($fields["placeholder"]) ? 'placeholder="'.trim($fields["placeholder"]).'"' : '';
        $output       	= sprintf( '<input type="text" id="%s" name="%s" '.$class.' '.$placeholder.' value="%s" />', esc_attr( $input ), esc_attr( $option ), esc_attr( $value ) );
        if( $class && $descript ) {
            $output   	.= sprintf( '<p class="description">%s</p>', $descript );
        }
        if( ! $class && $descript ) {
            $output   	.= sprintf( '<span class="description">%s</span>', $descript );
        }
        printf( $output );
	}

	public function wanzi_framework_checkbox_callback( $args ) {
        $input          = $args['id'];
        $data        	= $args['data'];
        $fields         = $args['field'];
		$option 		= $args['option'];
		$default        = $args["default"];
        $value          = $data ? $data : $default;
        $descript       = isset($fields["description"]) ? trim($fields["description"]) : '';
        $output       	= sprintf( '<input type="checkbox" id="%s" name="%s" '. checked( (int)$value, 1, false) .' value="1" />',  esc_attr( $input ), esc_attr( $option ) );
        if( $descript ) {
            $output   	.= sprintf( '<span class="description">%s</span>', $descript );
        }
        printf( $output );
    }

	public function wanzi_framework_radio_callback( $args ) {
        $input          = $args['id'];
        $data        	= $args['data'];
        $fields         = $args['field'];
		$option 		= $args['option'];
		$default        = $args["default"];
        $value          = $data ? $data : $default;
        $descript       = isset($fields["description"]) ? trim($fields["description"]) : '';
        $options        = isset($fields["options"]) ? trim($fields["options"]) : [];
		$output 		= '';
        foreach ( $options as $id => $_option ) {
            $output 	.= sprintf( '<input type="radio" id="%s" name="%s" '. checked( $value, $id, false ) .' value="%s" /><label for="%s">%s</label>',  esc_attr( $input ), esc_attr( $option ), esc_attr( $id ), esc_attr( $input.'_'.$id ), esc_html( $_option ) );
        }
        if( $descript ) {
            $output   	.= sprintf( '<p class="description">%s</p>', $descript );
        }
        printf( $output );
    }

	public function wanzi_framework_radio_i_callback( $args ) {
        $input          = $args['id'];
        $data        	= $args['data'];
        $fields         = $args['field'];
		$option 		= $args['option'];
		$default        = $args["default"];
        $value          = $data ? $data : $default;
        $descript       = isset($fields["description"]) ? trim($fields["description"]) : '';
        $options        = isset($fields["options"]) ? (array)$fields["options"] : [];
		$output 		= '';
        foreach ( $options as $id => $_option ) {
            $output 	.= sprintf( '<label class="normal-radio--label"><input class="normal-radio--radio" type="radio" id="%s" name="%s" '. checked( $value, $id, false ) .' value="%s" /><img class="normal-radio--image" src="%s" /></label>',  esc_attr( $input ), esc_attr( $option ), esc_attr( $id ), esc_html( $_option ) );
        }
        if( $descript ) {
            $output   	.= sprintf( '<p class="description">%s</p>', $descript );
        }
        printf( $output );
    }

	public function wanzi_framework_select_callback( $args ) {
        $input          = $args['id'];
        $data        	= $args['data'];
        $fields         = $args['field'];
		$option 		= $args['option'];
		$default        = $args["default"];
        $value          = $data ? $data : $default;
        $descript       = isset($fields["description"]) ? trim($fields["description"]) : '';
        $options        = isset($fields["options"]) ? (array)$fields["options"] : [];
        $output       	= '<select name="' .esc_attr( $option ). '" id="' . esc_attr( $input ) . '">';
        foreach ( $options as $id => $_option ) {
            $output   	.= '<option'. selected( $value, $id, false ) .' value="' . esc_attr( $id ) . '">' . esc_html( $_option ) . '</option>';
        }
        $output       	.= '</select>';
        if( $descript ) {
            $output   	.= sprintf( '<span class="description">%s</span>', $descript );
        }
        printf( $output );
    }

	public function wanzi_framework_password_callback( $args ) {
        $input          = $args['id'];
        $data        	= $args['data'];
        $fields         = $args['field'];
		$option 		= $args['option'];
		$default        = $args["default"];
        $value          = $data ? $data : $default;
        $class          = isset($fields["class"]) ? 'class="'.trim($fields["class"]).'"' : '';
        $descript       = isset($fields["description"]) ? trim($fields["description"]) : '';
        $placeholder    = isset($fields["placeholder"]) ? 'placeholder="'.trim($fields["placeholder"]).'"' : '';
        $output       	= sprintf( '<input type="password" id="%s" name="%s" '.$class.' '.$placeholder.' value="%s" />', esc_attr( $input ), esc_attr( $option ), esc_attr( $value ) );
        if( $class && $descript ) {
            $output   	.= sprintf( '<p class="description">%s</p>', $descript );
        }
        if( ! $class && $descript ) {
            $output   	.= sprintf( '<span class="description">%s</span>', $descript );
        }
        printf( $output );
    }
    
    public function wanzi_framework_number_callback( $args ) {
        $input          = $args['id'];
        $data        	= $args['data'];
        $fields         = $args['field'];
		$option 		= $args['option'];
		$default        = $args["default"];
        $value          = $data ? $data : $default;
        $step           = isset($fields["step"]) ? 'step="'.$fields["step"].'"' : '';
        $class          = isset($fields["class"]) ? 'class="'.trim($fields["class"]).'"' : '';
        $descript       = isset($fields["description"]) ? trim($fields["description"]) : '';
        $placeholder    = isset($fields["placeholder"]) ? 'placeholder="'.trim($fields["placeholder"]).'"' : '';
        $output       	= sprintf( '<input type="number" id="%s" name="%s" '.$class.' '.$step.' '.$placeholder.' value="%s" />', esc_attr( $input ), esc_attr( $option ), esc_attr( $value ) );
        if( $class && $descript ) {
            $output   	.= sprintf( '<p class="description">%s</p>', $descript );
        }
        if( ! $class && $descript ) {
            $output   	.= sprintf( '<span class="description">%s</span>', $descript );
        }
        printf( $output );
    }
    
    public function wanzi_framework_color_callback( $args ) {
        $input          = $args['id'];
        $option         = $args['option'];
        $fields         = $args['field'];
        $default        = $args["default"];
        $value          = $args["data"] ? $args["data"] : $default;
        $descript       = isset($fields["description"]) ? trim($fields["description"]) : '';
        $output       	= sprintf( '<input type="text" id="%s" name="%s" class="wp-color-result-text color-button" value="%s" />', esc_attr( $input ), esc_attr( $option ), esc_attr( $value ) );
        if( $descript ) {
            $output   	.= sprintf( '<span class="description">%s</span>', $descript );
        }
        printf( $output );
    }
    
    public function wanzi_framework_upload_callback( $args ) {
        $input          = $args['id'];
        $option         = $args['option'];
        $fields         = $args['field'];
        $default        = $args["default"];
        $value          = $args["data"] ? $args["data"] : $default;
        $class          = isset($fields["class"]) ? 'class="'.trim($fields["class"]).'"' : '';
        $descript       = isset($fields["description"]) ? trim($fields["description"]) : '';
        $placeholder    = isset($fields["placeholder"]) ? 'placeholder="'.trim($fields["placeholder"]).'"' : '';
        $output       	= sprintf( '<input type="text" id="%s" name="%s" '.$class.' '.$placeholder.' value="%s" />', esc_attr( $input ), esc_attr( $option ), esc_attr( $value ) );
        $output 		.= sprintf( '<input type="button" id="%s-btn" class="button upload-button" value="选择媒体">', esc_attr( $input ) );
		if( $descript ) {
            $output   .= sprintf( '<p class="description">%s</p>', $descript );
        }
        printf( $output );
    }

	public function wanzi_framework_textarea_callback( $args ) {
        $input          = $args['id'];
        $option         = $args['option'];
        $fields         = $args['field'];
        $default        = $args["default"];
        $value          = $args["data"] ? $args["data"] : $default;
        $rows           = isset($fields["rows"]) ? (int)$fields["rows"] : 4;
        $class          = isset($fields["class"]) ? 'class="'.trim($fields["class"]).'"' : '';
        $descript       = isset($fields["description"]) ? trim($fields["description"]) : '';
        $placeholder    = isset($fields["placeholder"]) ? 'placeholder="'.trim($fields["placeholder"]).'"' : '';
        $output       	= sprintf( '<textarea id="%s" name="%s" '.$class.' rows="'.$rows.'" '.$placeholder.'>%s</textarea>', esc_attr( $input ), esc_attr( $option ), esc_textarea( $value ) );
        if( $descript ) {
            $output   	.= sprintf( '<p class="description">%s</p>', $descript );
        }
        printf( $output );
    }

	public function wanzi_framework_editor_callback( $args ) {
        $input          = $args['id'];
        $option         = $args['option'];
        $fields         = $args['field'];
        $default        = $args["default"];
        $value          = $args["data"] ? $args["data"] : $default;
        $rows           = isset($fields["rows"]) ? (int)$fields["rows"] : 4;
        $class          = isset($fields["class"]) ? 'class="'.trim($fields["class"]).'"' : '';
        $descript       = isset($fields["description"]) ? trim($fields["description"]) : '';
        $editor         = isset($fields["setting"]) ? $fields["setting"] : ['media_buttons' => false, 'textarea_rows' => $rows, 'tinymce' => array( 'plugins' => 'wordpress,wplink' )];
        wp_editor( $value, $input, $editor );
        if( $descript ) {
            printf( '<p class="description">'.$descript.'</p>' );
        }
    }

	public function wanzi_framework_info_callback( $args ) {
        $input          = $args['id'];
        $option         = $args['option'];
        $fields         = $args['field'];
        $descript       = isset($fields["description"]) ? trim($fields["description"]) : '';
        $content        = isset($fields["content"]) ? apply_filters( 'the_content',$fields["content"] ) : $descript;
        printf( $content );
    }

	public function wanzi_framework_mu_check_callback( $args ) {
        $input          = $args['id'];
        $option         = $args['option'];
        $fields         = $args['field'];
        $default        = $args["default"];
        $value          = $args["data"] ? $args["data"] : $default;
        $descript       = isset($fields["description"]) ? trim($fields["description"]) : '';
        $output       	= '';
        if( isset($fields['options']) ) {
            foreach( $fields['options'] as $key => $_option ) {
                $data 		= isset($value[$key]) ? (int)$value[$key] : 0;
                $output 	.= sprintf( '<input id="%s" type="checkbox" name="%s" ' .checked( $data, 1, false ). ' value="1" /><span class="%s mu-mar">%s</span>', esc_attr( $key ), esc_attr( $option.'['.$key.']' ), esc_attr( $key ), esc_html( $_option ) );
            }
            if( $descript ) {
                $output 	.= sprintf( '<p class="description">%s</p>', $descript );
            }
        }
        printf( $output );
    }

	public function wanzi_framework_mu_text_callback( $args ) {
        $input          = $args['id'];
        $option         = $args['option'];
        $fields         = $args['field'];
        $default        = $args["default"];
        $value          = $args["data"] ? $args["data"] : $default;
        $class          = isset($fields["class"]) ? 'class="'.trim($fields["class"]).'"' : '';
        $placeholder    = isset($fields["placeholder"]) ? 'placeholder="'.trim($fields["placeholder"]).'"' : '';
        $output       	= '<div class="mu-texts sortable ui-sortable">';
        if( $value ) {
            foreach( $value as $_value ) {
                $output .= '<div class="mu-item">';
                $output .= sprintf( '<input id="%s" type="text" name="%s" '.$class.' '.$placeholder.' value="%s" />', esc_attr( $input ), esc_attr( $option.'[]' ), esc_html( $_value ) );
                $output .= '<a href="javascript:;" class="button del-item">删除</a>';
                $output .= '<span class="dashicons dashicons-menu ui-sortable-handle"></span>';
                $output .= '</div>';
            }
        }
        $output       	.= '<div class="mu-item">';
        $output       	.= sprintf( '<input id="%s" type="text" name="%s" '.$class.' '.$placeholder.' value="" />', esc_attr( $input ), esc_attr( $option.'[]' ) );
        $output       	.= '<a class="mp-mu-text button">添加选项</a>';
        $output       	.= '</div>';
        $output       	.= '</div>';
        printf( $output );
    }

    public function wanzi_option_framework_sanitize( $input ) {
        $data = array( );
        $options = $this->fields;
        if( $options ) {
            foreach ( $options as $key => $option ) {
                $fields = $option["fields"];
                foreach ( $fields as $var => $field ) {
                    if( ! isset( $var ) ) {
                        continue;
                    }
                    if( ! isset( $field['type'] ) ) {
                        continue;
                    }
                    $id = preg_replace( '/[^a-zA-Z0-9._\-]/', '', strtolower( $var ) );
                    if( 'checkbox' == $field['type'] && ! isset( $input[$id] ) ) {
                        $input[$id] = false;
                    }
                    if( 'mu-check' == $field['type'] && ! isset( $input[$id] ) ) {
                        foreach ( $field['options'] as $key => $value ) {
                            $input[$id][$key] = false;
                        }
                    }
                    if( 'mu-text' == $field['type'] && ! isset( $input[$id] ) ) {
                        $input[$id] = false;
                    }
                    if( has_filter( 'wanzi_sanitize_' . $field['type'] ) ) {
                        $data[$id] = apply_filters( 'wanzi_sanitize_' . $field['type'], $input[$id], $field );
                    } else {
                        $data[$id] = apply_filters( 'wanzi_sanitize_multi_field', $input[$id], $field );
                    }
                }
            }
        }
        do_action( 'wanzi_framework_sanitize_validate', $data, $this->option );
	    return $data;
    }

}