<?php

if ( !defined( 'ABSPATH' ) ) exit;

function wanzi_count_comment_type( $post_id = 0, $comment_type = 'comment' ) {
    if( ! $post_id ) {
        global $post;
        $post_id = $post->ID;
    }
    $args = array(
        'post_id'  => $post_id,
        'type__in' => array( $comment_type ),
        'count'    => true,
        'status'   => 'approve'
    );
    $counts = (int)get_comments( $args );
    $update = apply_filters( 'wanzi_comment_update', WP_DEBUG );
    $total  = (int)get_post_meta( $post_id, $comment_type.'s', true );
    if( $update && $total && $counts ) {
        return apply_filters( 'wanzi_count_comment', max($total, $counts), $post_id, $comment_type );
    }
    if( ! update_post_meta( $post_id, $comment_type.'s', $counts ) ) {
		add_post_meta( $post_id, $comment_type.'s', $counts, true );
	}
    return apply_filters( 'wanzi_count_comment', $counts, $post_id, $comment_type );
}

function wanzi_comment_post_status( $post_id = 0, $user_id = 0, $comment_type = 'comment' ) {
    if( ! $post_id ) {
        global $post;
        $post_id = $post->ID;
    }
    if( ! $user_id ) {
        $user_id = get_current_user_id( );
    }
    $user = get_user_by( 'ID', $user_id );
	if( ! $user ) {
		return 0;
	}
	$args = array(
        'post_id'  => $post_id,
        'type__in' => array( $comment_type ),
        'user_id'  => $user_id,
        'count'    => true,
        'status'   => 'approve'
    );
	$count = (int)get_comments( $args );
	return $count;
}

function wanzi_get_comments_by_type( $post_id = 0, $comment_type = 'comment', $number = 10 ) {
    if( ! $post_id ) {
        global $post;
        $post_id = $post->ID;
    }
    $args = array(
        'post_id'  => $post_id,
        'type__in' => array( $comment_type ),
        'number'   => $number,
        'status'   => 'approve'
    );
    $authors  = array( );
    $comments = get_comments( $args );
    foreach ( $comments as $comment ) {
		$_data   = array( );
		$user_id = $comment->user_id;
		$avatar  = get_user_meta( $user_id, 'avatar', true );
        if( ! $avatar ) {
            $avatar = get_avatar_url( $user_id );
        }
		$_data["id"]     = $user_id;
		$_data["name"]   = get_the_author_meta('nickname', $user_id);
		$_data["avatar"] = $avatar;
		$authors[] = $_data;
	}
    return apply_filters( 'wanzi_post_comment_users', $authors, $args );
}

function wanzi_get_user_comment_post_ids( $args = '' ) {
    global $wpdb;
	$query                    = "";
	$where                    = "";
    $limits                   = "";
    $_args                    = wp_parse_args( $args );
    if( isset($_args['user_id']) ) {
        $query .= $wpdb->prepare( "AND user_id = %s ", esc_sql( $_args['user_id'] ) );
	}
	if( isset($_args['status']) ) {
        $query .= $wpdb->prepare( "AND comment_approved = %s ", esc_sql( (int)$_args['status'] ) );
	}
	if( isset($_args['type']) ) {
        $query .= $wpdb->prepare( "AND comment_type = %s ", esc_sql( $_args['type'] ) );
	}
	if( isset($_args['parent']) ) {
        $query .= $wpdb->prepare( "AND 	comment_parent = %s ", esc_sql( (int)$_args['parent'] ) );
	}
	if( isset($_args['offset']) && isset($_args['number']) ) {
        $offset = (int)$_args['offset'];
        $per_page = (int)$_args['number'];
        if( $offset && $per_page ) {
            $limits = "LIMIT $offset, $per_page";
        } else if( !$offset && $per_page ) {
            $limits = "LIMIT $per_page";
        }
    }
    $orderby = "ORDER BY MAX(comment_date) DESC";
	$groupby = "GROUP BY comment_post_ID";
	if( $query ) {
        $where = "WHERE $query";
    }
    $sql 	= "SELECT comment_post_ID as post_id, MAX(comment_date) as date FROM $wpdb->comments $where $groupby $orderby $limits";
	$sql    = @str_replace( 'WHERE AND', 'WHERE', $sql );
    $result = $wpdb->get_results( $sql );
    return $result;
}


function wanzi_cover_for_qrcode_posters( $thumbnail ) {
	$_domains = array( );
	$urls = parse_url( $thumbnail );
	$domains = wp_miniprogram_option('trust_domain');
	foreach( $domains as $domain ) {
		$_domains[] = str_replace( "http://", "", str_replace( "https://", "", $domain ) );
	}
	if( in_array($urls["host"], $_domains) ) { 
		return apply_filters( 'wanzi_posters_cover', $thumbnail );
	}
	return apply_filters( 'wanzi_posters_cover', wp_miniprogram_option('thumbnail') );
}

function wanzi_comment_type_in( $type ) {
	$args = array( "like" => "点赞", "fav" => "收藏" );
	$option = apply_filters( 'wanzi_comment_type_option', $args );
	if( array_key_exists( $type, $option ) ) {
		return true;
	}
	return false;
}

function wanzi_comment_type_lable( $type ) {
	$option = array( "like" => "点赞", "fav" => "收藏" );
	$title = apply_filters( 'custom_comment_type_text', $option );
	if( isset($title[$type]) ) {
		return $title[$type];
	}
	return $type;
}

function wanzi_comment_post_lable( $post_id ) {
	$option = array( "post" => "文章" );
	$title = apply_filters( 'custom_comment_post_text', $option );
	$type = get_post_type( $post_id );
	if( isset($title[$type]) ) {
		return $title[$type];
	}
	return $type;
}

function wanzi_parse_tencent_video( $url ) {
    if( filter_var($url, FILTER_VALIDATE_URL) ) { 
		if(preg_match('#https://v.qq.com/x/page/(.*?).html#i',$url, $m)) {
			$url = $m[1];
		} else if(preg_match('#https://v.qq.com/x/cover/.*/(.*?).html#i',$url, $m)) {
			$url = $m[1];
		}
	}
	if( strlen( $url ) > 20 ) {
		return $url;
	}
	$api = 'https://vv.video.qq.com/getinfo?vid='.$url.'&platform=101001&charge=0&otype=json';
	$remote = wp_remote_get( $api );
	if( is_wp_error( $remote ) ) {
	    return $url;
	}
	$response = wp_remote_retrieve_body( $remote );
	$response = substr($response,13,-1);
	$response = json_decode($response,true);
	$response = $response['vl']['vi'][0];
	$mp4file  = $response['fn'];
	$mp4keys  = $response['fvkey'];
	$mp4	  = 'https://ugcws.video.gtimg.com/'.$mp4file.'?vkey='.$mp4keys;
	return $mp4;
}

function wanzi_get_reply_comments( $post_id, $user, $parent ) {
    $args = array(
		'post_id'  => $post_id,
		'type__in' => array('comment'),
		'status'   => 'approve',
		'parent'   => $parent,
		'number'   => 10,
		"orderby"  => 'comment_date',
		"order"    => 'DESC'
	);
	$comments = get_comments($args);
	$data     = array( );
	foreach( $comments as $comment ) {
		$comment_id                     = $comment->comment_ID;
		$user_id                        = $comment->user_id;
		$user_name                      = $comment->comment_author;
		$date                           = $comment->comment_date;
		$content                        = $comment->comment_content;
		$parent                         = $comment->comment_parent;
		$avatar                         = get_user_meta( $user_id, 'avatar', true );
		$_data["id"]                    = $comment_id;
		$_data["author"]["id"]          = $user_id;
		$_data["author"]["name"]        = ucfirst( $user_name );
		if ($avatar) {
			$_data["author"]["avatar"]  = $avatar;
		} else {
			$_data["author"]["avatar"]  = get_avatar_url( $user_id );
		}
		$_data["date"]                  = datetime_before( $date );
		$_data["content"]               = $content;
		$_data["parent"]                = $parent;
		$_data["reply_to"]              = ucfirst( $user );
		$_data["reply"]                 = wanzi_get_reply_comments( $post_id, $user_name, $comment_id );
		$data[]                         = apply_filters( 'wanzi_rest_reply_comment', $_data, $comment );
	}
	return $data;
}

function wanzi_weixin_security_msg_check( $content ) {
    $token = MP_Auth::we_miniprogram_access_token( );
	if( !isset($token['access_token']) || empty($token['access_token']) ) {
		return new WP_Error( 'error', 'access token 错误' , array( 'status' => 403 ) );
	}
	$url = 'https://api.weixin.qq.com/wxa/msg_sec_check?access_token='.trim($token['access_token']);
    $header = array(
        "Content-Type: application/json;charset=UTF-8"
    );
	$msg  = wp_strip_all_tags( $content );
	$body = '{"content":"'.$msg.'"}';
	$args = array(
		'method'  => 'POST',
		'body' 	  => $body,
		'headers' => $header,
		'cookies' => array( )
	);
	$response = wp_remote_post( $url, $args );
	$result   = wp_remote_retrieve_body( $response );
	return json_decode( $result );
}