<?php

if( !defined( 'ABSPATH' ) ) exit;

class WP_REST_Custom_Router extends WP_REST_Controller {
    
    public function __construct( ) {
		$this->namespace     = 'mp/v1';
        $this->rest_base     = 'posts';
        $this->meta          = new WP_REST_Post_Meta_Fields( 'post' );
	}

	public function register_routes( ) {
		
		register_rest_route( $this->namespace, '/' . $this->rest_base.'/most/(?P<slug>[\w-]+)', array(
		    'args' => array(
				'slug' => array(
					'description'  => __( 'Unique identifier for the object' ),
					'type'         => 'string'
				)
			),
			array(
				'methods'             	=> WP_REST_Server::READABLE,
				'callback'            	=> array( $this, 'get_most_posts' ),
				'permission_callback' 	=> array( $this, 'get_items_permissions_check' ),
				'args'                	=> array(
				    'context'  => $this->get_context_param( array( 'default' => 'view' ) )
				)
			)
		) );
		
		register_rest_route( $this->namespace, '/' . $this->rest_base.'/relate/(?P<id>[\d]+)', array(
		    'args' => array(
    		    'id' => array(
    			    'description' => __( 'Unique identifier for the object.' ),
    				'type'        => 'integer',
    			),
    		),
			array(
				'methods'             	=> WP_REST_Server::READABLE,
				'callback'            	=> array( $this, 'get_relate_posts' ),
				'permission_callback' 	=> array( $this, 'get_items_permissions_check' ),
				'args'                	=> array(
				    'context'  => $this->get_context_param( array( 'default' => 'view' ) )
				)
			)
		) );

		register_rest_route( $this->namespace, '/' . $this->rest_base.'/comment', array(
			array(
				'methods'             	=> WP_REST_Server::READABLE,
				'callback'            	=> array( $this, 'get_comment_posts' ),
				'permission_callback' 	=> array( $this, 'get_items_permissions_check' ),
				'args'                	=> array(
				    'context'  => $this->get_context_param( array( 'default' => 'view' ) )
				)
			)
		) );

		register_rest_route( $this->namespace, '/' . $this->rest_base.'/meta', array(
			array(
				'methods'             	=> WP_REST_Server::READABLE,
				'callback'            	=> array( $this, 'get_meta_posts' ),
				'permission_callback' 	=> array( $this, 'get_items_permissions_check' ),
				'args'                	=> array(
				    'context'  => $this->get_context_param( array( 'default' => 'view' ) )
				)
			)
		) );

		$schema = $this->get_item_schema( );
		
	}

	public function get_items_permissions_check( $request ) {

		return true;

	}
	
	public function get_most_posts( $request ) {
	    
	    $args   = array( );
	    
	    $params = array(
			'author'         => 'author__in',
			'author_exclude' => 'author__not_in',
			'exclude'        => 'post__not_in',
			'include'        => 'post__in',
			'offset'         => 'offset',
			'order'          => 'order',
			'orderby'        => 'orderby',
			'page'           => 'paged',
			'parent'         => 'post_parent__in',
			'parent_exclude' => 'post_parent__not_in',
			'search'         => 's',
			'slug'           => 'post_name__in',
			'per_page'       => 'posts_per_page'
		);
		
		foreach ( $params as $api_param => $wp_param ) {
		    if( isset( $request[ $api_param ] ) ) {
		        $args[ $wp_param ] = $request[ $api_param ];
		    }
		}
		
		$meta = isset($request['meta']) ? trim($request['meta']) : 'views';
		if( ! preg_match("/^[a-zA-Z\s]+$/", $meta) ) {
			new WP_Error( 'rest_post_invalid_meta_key', __( 'Invalid post meta_key.' ), array( 'status' => 404 ) );
		}
		
		$args['meta_key']   = $meta;
		$args['date_query'] = array( );
		
		if ( isset( $request['before'] ) ) {
			$args['date_query'][] = array(
				'before' => $request['before'],
				'column' => 'post_date',
			);
		}
		
		if ( isset( $request['after'] ) ) {
			$args['date_query'][] = array(
				'after'  => $request['after'],
				'column' => 'post_date',
			);
		}
		
		$args['post_type'] = 'post';
		
		$query_args = apply_filters( "wanzi_most_post_api_query", $args, $request );
		
		if ( ! isset( $query_args['orderby'] ) ) {
		    $query_args['orderby'] = 'meta_value_num';
		}
		
		if ( ! isset( $query_args['order'] ) ) {
		    $query_args['order'] = 'DESC';
		}
		
		if( empty( $query_args['date_query'] ) ) {
		    $query_args['date_query'][] = ['after' => '1 year ago'];
		}
		
		$query_args['update_post_meta_cache'] = false;
		
		$taxonomies = wp_list_filter( get_object_taxonomies( 'post', 'objects' ), array( 'show_in_rest' => true ) );

		if( ! empty( $request['tax_relation'] ) ) {
			$query_args['tax_query'] = array( 'relation' => $request['tax_relation'] );
		}
		
		foreach ( $taxonomies as $taxonomy ) {
			$base = ! empty( $taxonomy->rest_base ) ? $taxonomy->rest_base : $taxonomy->name;
			$tax_exclude = $base . '_exclude';

			if( ! empty( $request[ $base ] ) ) {
				$query_args['tax_query'][] = array(
					'taxonomy'         => $taxonomy->name,
					'field'            => 'term_id',
					'terms'            => $request[ $base ],
					'include_children' => false,
				);
			}

			if( ! empty( $request[ $tax_exclude ] ) ) {
				$query_args['tax_query'][] = array(
					'taxonomy'         => $taxonomy->name,
					'field'            => 'term_id',
					'terms'            => $request[ $tax_exclude ],
					'include_children' => false,
					'operator'         => 'NOT IN',
				);
			}
		}
		
		$posts        = array( );
		$posts_query  = new WP_Query( );
		$query_result = $posts_query->query( $query_args );

		foreach ( $query_result as $post ) {
			if( ! $this->check_read_permission( $post ) ) {
				continue;
			}

			$data    = $this->prepare_item_for_response( $post, $request );
			$posts[] = $this->prepare_response_for_collection( $data );
		}

		$response  = rest_ensure_response( $posts );
		$response->header( 'Cache-Control', 'max-age=3600' );
		
		return $response;
		
	}
	
	public function get_relate_posts( $request ) {
	    
	    $post_id = (int)$request["id"];
	    if( $post_id <= 0 ) {
		    return new WP_Error( 'rest_relate_id_error', __( '相关文章 ID 无效.' ), array( 'status' => 400 ) );
		}
		$post = get_post( $post_id );
		if( is_wp_error( $post ) || empty( $post->ID ) ) {
			return new WP_Error( 'rest_relate_id_error', __( '相关文章 ID 无效.' ), array( 'status' => 400 ) );
		}
		$args   = array( );
	    $params = array(
			'author'         => 'author__in',
			'author_exclude' => 'author__not_in',
			'exclude'        => 'post__not_in',
			'include'        => 'post__in',
			'offset'         => 'offset',
			'order'          => 'order',
			'orderby'        => 'orderby',
			'page'           => 'paged',
			'parent'         => 'post_parent__in',
			'parent_exclude' => 'post_parent__not_in',
			'search'         => 's',
			'slug'           => 'post_name__in',
			'per_page'       => 'posts_per_page'
		);
		
		foreach ( $params as $api_param => $wp_param ) {
		    if( isset( $request[ $api_param ] ) ) {
		        $args[ $wp_param ] = $request[ $api_param ];
		    }
		}
		
		$args['date_query'] = array( );
		
		if ( isset( $request['before'] ) ) {
			$args['date_query'][] = array(
				'before' => $request['before'],
				'column' => 'post_date',
			);
		}
		
		if ( isset( $request['after'] ) ) {
			$args['date_query'][] = array(
				'after'  => $request['after'],
				'column' => 'post_date',
			);
		}
		
		$args['post_type'] 		= $post->post_type;
		$args['post__not_in'][] = $post_id;
		
		$query_args = apply_filters( "wanzi_relate_post_api_query", $args, $request );
		
		if ( ! isset( $query_args['orderby'] ) ) {
		    $query_args['orderby'] = 'date';
		}
		
		if ( ! isset( $query_args['order'] ) ) {
		    $query_args['order'] = 'DESC';
		}
		
		if( empty( $query_args['date_query'] ) ) {
		    $query_args['date_query'][] = ['after' => '1 year ago'];
		}
		
		$query_args['update_post_meta_cache'] = false;
		
		$taxonomies = wp_list_filter( get_object_taxonomies( $post->post_type, 'objects' ), array( 'show_in_rest' => true ) );

		if( ! empty( $request['tax_relation'] ) ) {
			$query_args['tax_query'] = array( 'relation' => $request['tax_relation'] );
		}
		
		foreach ( $taxonomies as $taxonomy ) {
			$base = ! empty( $taxonomy->rest_base ) ? $taxonomy->rest_base : $taxonomy->name;
			$tax_exclude = $base . '_exclude';

			if( ! empty( $request[ $base ] ) ) {
				$query_args['tax_query'][] = array(
					'taxonomy'         => $taxonomy->name,
					'field'            => 'term_id',
					'terms'            => $request[ $base ],
					'include_children' => false,
				);
			}

			if( ! empty( $request[ $tax_exclude ] ) ) {
				$query_args['tax_query'][] = array(
					'taxonomy'         => $taxonomy->name,
					'field'            => 'term_id',
					'terms'            => $request[ $tax_exclude ],
					'include_children' => false,
					'operator'         => 'NOT IN',
				);
			}
		}
		
		$tags = get_the_tags( $post_id );
		if( ! empty( $tags ) ) {
    		foreach( $tags as $tag ) {
    			$query_args['tag__in'][] = $tag->term_id;
    		}
		}
		
		$posts        = array( );
		$posts_query  = new WP_Query( );
		$query_result = $posts_query->query( $query_args );

		foreach ( $query_result as $post ) {
			if( ! $this->check_read_permission( $post ) ) {
				continue;
			}

			$data    = $this->prepare_item_for_response( $post, $request );
			$posts[] = $this->prepare_response_for_collection( $data );
		}
		
		$response  = rest_ensure_response( $posts );
		$response->header( 'Cache-Control', 'max-age=3600' );
		
		return $response;
		
	}

	public function get_comment_posts( $request ) {
	    
	    $params = array(
			'author'         => 'author__in',
			'author_exclude' => 'author__not_in',
			'exclude'        => 'post__not_in',
			'include'        => 'post__in',
			'offset'         => 'offset',
			'order'          => 'order',
			'orderby'        => 'orderby',
			'page'           => 'paged',
			'parent'         => 'post_parent__in',
			'parent_exclude' => 'post_parent__not_in',
			'search'         => 's',
			'slug'           => 'post_name__in',
			'per_page'       => 'posts_per_page'
		);
		
		foreach ( $params as $api_param => $wp_param ) {
		    if( isset( $request[ $api_param ] ) ) {
		        $args[ $wp_param ] = $request[ $api_param ];
		    }
		}
		
		
		$type           = isset($request["type"]) ? $request["type"] : "comment";
		$page           = isset($args["paged"]) ? $args["paged"] : 1;
		$per_page       = isset($args["posts_per_page"]) ? $args["posts_per_page"] : 10;
		$offset         = ( $page * $per_page ) - $per_page;
		$access_token   = isset($request['access_token']) ? $request['access_token'] : "";
		$session        = base64_decode( $access_token );
		$users          = MP_Auth::login( $session );
		if ( !$users ) {
			return new WP_Error( 'access_token_error', '授权信息有误' , array( 'status' => 403 ) );
		}
		
		$comments_args  = array( 
			'type'      => $type, 
			'status'    => 1, 
			'user_id'   => $users->ID, 
			'number'    => $per_page, 
			'offset'    => $offset 
		);
		$comment_posts  = wanzi_get_user_comment_post_ids( $comments_args );
		if( empty($comment_posts) ) {
		    return rest_ensure_response( [] );
		}
		foreach ( $comment_posts as $post ) {
		    $args['post__in'][] = $post->post_id;
		}
		
		$args['date_query'] = array( );
		
		if ( isset( $request['before'] ) ) {
			$args['date_query'][] = array(
				'before' => $request['before'],
				'column' => 'post_date',
			);
		}
		
		if ( isset( $request['after'] ) ) {
			$args['date_query'][] = array(
				'after'  => $request['after'],
				'column' => 'post_date',
			);
		}
		
		$args['post_type'] = 'post';
		
		$query_args = apply_filters( "wanzi_{$type}_post_api_query", $args, $request );
		
		$query_args['update_post_meta_cache'] = false;
		
		$taxonomies = wp_list_filter( get_object_taxonomies( 'post', 'objects' ), array( 'show_in_rest' => true ) );

		if( ! empty( $request['tax_relation'] ) ) {
			$query_args['tax_query'] = array( 'relation' => $request['tax_relation'] );
		}
		
		foreach ( $taxonomies as $taxonomy ) {
			$base = ! empty( $taxonomy->rest_base ) ? $taxonomy->rest_base : $taxonomy->name;
			$tax_exclude = $base . '_exclude';

			if( ! empty( $request[ $base ] ) ) {
				$query_args['tax_query'][] = array(
					'taxonomy'         => $taxonomy->name,
					'field'            => 'term_id',
					'terms'            => $request[ $base ],
					'include_children' => false,
				);
			}

			if( ! empty( $request[ $tax_exclude ] ) ) {
				$query_args['tax_query'][] = array(
					'taxonomy'         => $taxonomy->name,
					'field'            => 'term_id',
					'terms'            => $request[ $tax_exclude ],
					'include_children' => false,
					'operator'         => 'NOT IN',
				);
			}
		}
		
		$posts        = array( );
		$posts_query  = new WP_Query( );
		$query_result = $posts_query->query( $query_args );

		foreach ( $query_result as $post ) {
			if( ! $this->check_read_permission( $post ) ) {
				continue;
			}

			$data    = $this->prepare_item_for_response( $post, $request );
			$posts[] = $this->prepare_response_for_collection( $data );
		}
		
		$response  = rest_ensure_response( $posts );
		$response->header( 'Cache-Control', 'max-age=3600' );
		
		return $response;
		
	}

	public function get_meta_posts( $request ) {
	    $args   = array( );
	    $params = array(
			'author'         => 'author__in',
			'author_exclude' => 'author__not_in',
			'exclude'        => 'post__not_in',
			'include'        => 'post__in',
			'offset'         => 'offset',
			'order'          => 'order',
			'orderby'        => 'orderby',
			'page'           => 'paged',
			'parent'         => 'post_parent__in',
			'parent_exclude' => 'post_parent__not_in',
			'search'         => 's',
			'slug'           => 'post_name__in',
			'per_page'       => 'posts_per_page',
			'key'            => 'meta_key',
			'value'          => 'meta_value',
			'compare'        => 'meta_compare'
		);
		
		foreach ( $params as $api_param => $wp_param ) {
		    if( isset( $request[ $api_param ] ) ) {
		        $args[ $wp_param ] = $request[ $api_param ];
		    }
		}
		
		$args['date_query'] = array( );
		
		if ( isset( $request['before'] ) ) {
			$args['date_query'][] = array(
				'before' => $request['before'],
				'column' => 'post_date',
			);
		}
		
		if ( isset( $request['after'] ) ) {
			$args['date_query'][] = array(
				'after'  => $request['after'],
				'column' => 'post_date',
			);
		}
		
		$meta_query = array( );
		$meta_args  = array( );
		
		if ( isset( $args['meta_key'] ) ) {
		    $meta_args['key'] = trim( $args['meta_key'] );
		}
		if ( isset( $args['meta_value'] ) ) {
		    $meta_args['value'] = trim( $args['meta_value'] );
		}
		if ( isset( $args['meta_compare'] ) ) {
		    $meta_args['compare'] = trim( $args['meta_compare'] );
		}
		
		if ( ! empty( $meta_args ) ) {
		    $meta_query[] = $meta_args;
		    unset($args['meta_key'], $args['meta_value'], $args['meta_compare']);
		}
		
		if ( isset( $request['query'] ) && ! empty( $meta_args ) ) {
		    $metas = json_decode( $request['query'], true );
		    if ( ! empty( $metas ) && is_array( $metas ) ) {
		        foreach ( $metas as $meta ) {
		            $meta_query[] = $meta;
		        }
		    }
		    if( isset( $request['relation'] ) ) {
		        $meta_query['relation'] = trim( $request['relation'] );
		    }
		}
		
		if ( isset( $request['query'] ) && empty( $meta_args ) ) {
		    $metas = json_decode( $request['query'], true );
		    if ( ! empty( $metas ) && is_array( $metas ) ) {
		        $meta_query = $metas;
		    }
		}
		
		if ( empty( $meta_query ) ) {
		    return new WP_Error( 'rest_meta_key_error', __( '文章自定义参数非法.' ), array( 'status' => 400 ) );
		}
		
		$args['meta_query'] = $meta_query;
		
		$args['post_type'] = 'post';
		
		$query_args = apply_filters( "wanzi_meta_key_api_query", $args, $request );
		
		$query_args['update_post_meta_cache'] = false;
		
		$taxonomies = wp_list_filter( get_object_taxonomies( 'post', 'objects' ), array( 'show_in_rest' => true ) );

		if( ! empty( $request['tax_relation'] ) ) {
			$query_args['tax_query'] = array( 'relation' => $request['tax_relation'] );
		}
		
		foreach ( $taxonomies as $taxonomy ) {
			$base = ! empty( $taxonomy->rest_base ) ? $taxonomy->rest_base : $taxonomy->name;
			$tax_exclude = $base . '_exclude';

			if( ! empty( $request[ $base ] ) ) {
				$query_args['tax_query'][] = array(
					'taxonomy'         => $taxonomy->name,
					'field'            => 'term_id',
					'terms'            => $request[ $base ],
					'include_children' => false,
				);
			}

			if( ! empty( $request[ $tax_exclude ] ) ) {
				$query_args['tax_query'][] = array(
					'taxonomy'         => $taxonomy->name,
					'field'            => 'term_id',
					'terms'            => $request[ $tax_exclude ],
					'include_children' => false,
					'operator'         => 'NOT IN',
				);
			}
		}
		
		$posts        = array( );
		$posts_query  = new WP_Query( );
		$query_result = $posts_query->query( $query_args );

		foreach ( $query_result as $post ) {
			if( ! $this->check_read_permission( $post ) ) {
				continue;
			}

			$data    = $this->prepare_item_for_response( $post, $request );
			$posts[] = $this->prepare_response_for_collection( $data );
		}
		
		$response  = rest_ensure_response( $posts );
		$response->header( 'Cache-Control', 'max-age=3600' );
		
		return $response;
		
	}
	
	public function can_access_password_content( $post, $request ) {
		if( empty( $post->post_password ) ) {
			return false;
		}

		if( empty( $request['password'] ) ) {
			return false;
		}

		return hash_equals( $post->post_password, $request['password'] );
	}
	
	protected function check_is_post_type_allowed( $post_type ) {
		if( ! is_object( $post_type ) ) {
			$post_type = get_post_type_object( $post_type );
		}

		if( ! empty( $post_type ) ) {
			return true;
		}

		return false;
	}

	public function check_read_permission( $post ) {
		$post_type = get_post_type_object( $post->post_type );
		if( ! $this->check_is_post_type_allowed( $post_type ) ) {
			return false;
		}

		if( 'publish' === $post->post_status || current_user_can( $post_type->cap->read_post, $post->ID ) ) {
			return true;
		}

		$post_status_obj = get_post_status_object( $post->post_status );
		if( $post_status_obj && $post_status_obj->public ) {
			return true;
		}

		if( 'inherit' === $post->post_status && $post->post_parent > 0 ) {
			$parent = get_post( $post->post_parent );
			if( $parent ) {
				return $this->check_read_permission( $parent );
			}
		}

		if( 'inherit' === $post->post_status ) {
			return true;
		}

		return false;
	}

	public function prepare_item_for_response( $post, $request ) {

		// 需要设置全局变量 上下篇文章必须
		$GLOBALS['post'] = $post;
		// 设置全局发布数据 上下篇文章必须
		setup_postdata( $post );

		$fields = $this->get_fields_for_response( $request );

		$data = array( );

		if( in_array( 'id', $fields, true ) ) {
			$data['id'] = $post->ID;
		}
		
		if( '0000-00-00 00:00:00' === $post->post_date_gmt ) {
			$post_date_gmt = get_gmt_from_date( $post->post_date );
		} else {
			$post_date_gmt = $post->post_date_gmt;
		}
		$data['date'] = date('Y-m-d', strtotime($post->post_date));
		$data['time'] = date('H:i:s', strtotime($post->post_date));
		$data["week"] = get_wp_post_week( $post->post_date );
		
		if( in_array( 'password', $fields, true ) ) {
			$data['password'] = $post->post_password;
		}

		if( in_array( 'slug', $fields, true ) ) {
			$data['slug'] = $post->post_name;
		}

		if( in_array( 'status', $fields, true ) ) {
			$data['status'] = $post->post_status;
		}

		if( in_array( 'type', $fields, true ) ) {
			$data['type'] = $post->post_type;
		}

		if( in_array( 'link', $fields, true ) ) {
			$data['link'] = get_permalink( $post->ID );
		}

		if( in_array( 'format', $fields, true ) ) {
			$data['format'] = get_post_format( $post->ID );

			if( empty( $data['format'] ) ) {
				$data['format'] = 'standard';
			}
		}

		if( in_array( 'meta', $fields, true ) ) {
			$data['meta']               = $this->meta->get_value( $post->ID, $request );
			$data['meta']["thumbnail"] 	= wanzi_post_thumbnail( $post->ID );
    		$data['meta']["views"] 		= (int)get_post_meta( $post->ID, "views" ,true );
    		$data['meta']["count"] 		= wanzi_text_mb_strlen( wp_strip_all_tags( $post->post_content ) );
    		if( get_post_meta( $post->ID, "source" ,true ) ) {
    			$_data["meta"]["source"] 	= get_post_meta( $post->ID, "source" ,true );
    		}
		}

		if( in_array( 'title', $fields, true ) ) {
			add_filter( 'protected_title_format', array( $this, 'protected_title_format' ) );

			$data['title'] = array(
				'raw'      => $post->post_title,
				'rendered' => html_entity_decode( get_the_title( $post->ID ) ),
			);

			remove_filter( 'protected_title_format', array( $this, 'protected_title_format' ) );
		}

		$user_id = 0;
		if( isset( $request['access_token'] ) ) {
			$access_token = $request['access_token'];
			$users = MP_Auth::login( base64_decode($access_token) );
			$user_id = (int)$users->ID;
		}

		$data['user_id'] = $user_id;
		
    	$author_id = $post->post_author;
    	$author_avatar = get_user_meta( $author_id, 'avatar', true );
    	$description = get_the_author_meta( 'description', $author_id );
    	$data['author'] = array(
    		'id' => ( int )$author_id,
    		'name' => get_the_author_meta( 'nickname', $author_id ),
    		'avatar' => $author_avatar ? $author_avatar : get_avatar_url( $author_id ),
    		'description' => $description ? $description : "这个家伙太懒，连个签名都没有"
    	);
    	
    	$data["comments"] 			= wanzi_count_comment_type( $post->ID, 'comment' );
		$data["isfav"] 				= (bool)wanzi_comment_post_status( $post->ID, $user_id, 'fav' );
		$data["favs"] 				= wanzi_count_comment_type( $post->ID, 'fav' );
		$data["islike"] 			= (bool)wanzi_comment_post_status( $post->ID, $user_id, 'like' );
		$data["likes"] 				= wanzi_count_comment_type( $post->ID, 'like' );
		
		$taxonomies = wp_list_filter( get_object_taxonomies( 'post', 'objects' ), array( 'show_in_rest' => true ) );

		foreach ( $taxonomies as $taxonomy ) {
			$base = ! empty( $taxonomy->rest_base ) ? $taxonomy->rest_base : $taxonomy->name;

			if( in_array( $base, $fields, true ) ) {
				$terms = get_the_terms( $post, $taxonomy->name );
				$data[ $base ] = array( );
				if( $terms ) {
					foreach( $terms as $term ) {
						$data[ $base ][] = array(
							'id' => $term->term_id,
							'name' => $term->name,
							'description' => $term->description,
							'cover' => get_term_meta($term->term_id,'cover',true)
						);
					}
				}
			}
		}
		
		if( wp_miniprogram_option('bd_appkey') && wp_miniprogram_option('bd_secret') ) {
		    $keywords 		= get_post_meta( $post->ID, "keywords", true );
			if( ! $keywords ) {
				$tags = wp_get_post_tags( $post->ID );
				$keywords 	= implode(",", wp_list_pluck( $tags, 'name' ));
			}
    		$data["smartprogram"]["title"] 		    = html_entity_decode( get_the_title( $post->ID ) ) .'-'.get_bloginfo('name');
    		$data["smartprogram"]["keywords"] 		= $keywords;
    		$data["smartprogram"]["description"] 	= wp_strip_all_tags( wp_trim_excerpt( "", $post->ID ), true );
    		$data["smartprogram"]["image"] 		    = wanzi_post_gallery( $post->ID );
    		$data["smartprogram"]["visit"] 		    = array( 'pv' => (int)get_post_meta( $post->ID, "views" ,true ) );
    		$data["smartprogram"]["comments"] 		= wanzi_count_comment_type( $post->ID, 'comment' );
    		$data["smartprogram"]["likes"] 		    = wanzi_count_comment_type( $post->ID, 'like' );
    		$data["smartprogram"]["collects"] 		= wanzi_count_comment_type( $post->ID, 'fav' );
		}

		$has_password_filter = false;

		if( $this->can_access_password_content( $post, $request ) ) {

			add_filter( 'post_password_required', '__return_false' );

			$has_password_filter = true;

		}

		if( in_array( 'excerpt', $fields, true ) ) {

			$the_excerpt = apply_filters( 'the_excerpt', $post->post_excerpt );
			if( ! $the_excerpt ) {
			    $the_excerpt = wp_trim_excerpt( "", $post->ID );
			}
			$data['excerpt'] = array(
				'raw'       => $post->post_excerpt,
				'rendered'  => html_entity_decode( wp_trim_words( wp_strip_all_tags( $the_excerpt, true ), 80 ) ),
				'protected' => (bool) $post->post_password
			);

		}

		if( in_array( 'content', $fields, true ) ) {
			$data['content'] = array(
				'raw'       => $post->post_content,
				'rendered'  => post_password_required( $post ) ? apply_filters( 'the_content', $post->post_excerpt ) : apply_filters( 'the_content', $post->post_content ),
				'protected' => (bool) $post->post_password
			);
		}

		if( $has_password_filter ) {
			remove_filter( 'post_password_required', '__return_false' );
		}

		$context = ! empty( $request['context'] ) ? $request['context'] : 'view';
		$data    = $this->add_additional_fields_to_object( $data, $request );
		$data    = $this->filter_response_by_context( $data, $context );

		$response = rest_ensure_response( $data );
		$response = apply_filters( "wanzi_rest_post", $response, $post, $request );

		return apply_filters( "rest_prepare_post", $response, $post, $request );
	}
	
	public function get_item_schema( ) {

		$schema = array(
			'$schema'    => 'http://json-schema.org/draft-04/schema#',
			'title'      => 'post',
			'type'       => 'object',
			'properties' => array(
				'date'            => array(
					'description' => __( "The date the object was published, in the site's timezone." ),
					'type'        => 'string',
					'format'      => 'date-time',
					'context'     => array( 'view', 'edit', 'embed' ),
				),
				'id'              => array(
					'description' => __( 'Unique identifier for the object.' ),
					'type'        => 'integer',
					'context'     => array( 'view', 'edit', 'embed' ),
					'readonly'    => true,
				),
				'link'            => array(
					'description' => __( 'URL to the object.' ),
					'type'        => 'string',
					'format'      => 'uri',
					'context'     => array( 'view', 'edit', 'embed' ),
					'readonly'    => true,
				),
				'slug'            => array(
					'description' => __( 'An alphanumeric identifier for the object unique to its type.' ),
					'type'        => 'string',
					'context'     => array( 'view', 'edit', 'embed' ),
					'arg_options' => array(
						'sanitize_callback' => array( $this, 'sanitize_slug' ),
					),
				),
				'status'          => array(
					'description' => __( 'A named status for the object.' ),
					'type'        => 'string',
					'enum'        => array_keys( get_post_stati( array( 'internal' => false ) ) ),
					'context'     => array( 'view', 'edit' ),
				),
				'type'            => array(
					'description' => __( 'Type of Post for the object.' ),
					'type'        => 'string',
					'context'     => array( 'view', 'edit', 'embed' ),
					'readonly'    => true,
				),
				'format'            => array(
					'description' => __( 'Format of Post for the object.' ),
					'type'        => 'string',
					'context'     => array( 'view', 'edit', ),
					'readonly'    => true,
				),
				'password'        => array(
					'description' => __( 'A password to protect access to the content and excerpt.' ),
					'type'        => 'string',
					'context'     => array( 'edit' ),
				),
			),
		);

		$schema['properties']['parent'] = array(
			'description' => __( 'The ID for the parent of the object.' ),
			'type'        => 'integer',
			'context'     => array( 'view', 'edit' ),
		);

		$post_type_attributes = array(
			'title',
			'editor',
			'author',
			'excerpt',
			'thumbnail',
			'comments',
			'revisions',
			'page-attributes',
			'post-formats',
			'custom-fields',
		);
		$fixed_schemas = array(
			'post' => array(
				'title',
				'editor',
				'author',
				'excerpt',
				'thumbnail',
				'comments',
				'revisions',
				'post-formats',
				'custom-fields',
			)
		);
		foreach ( $post_type_attributes as $attribute ) {
			if( isset( $fixed_schemas[ 'post' ] ) && ! in_array( $attribute, $fixed_schemas[ 'post' ], true ) ) {
				continue;
			} elseif( ! isset( $fixed_schemas[ 'post' ] ) && ! post_type_supports( 'post', $attribute ) ) {
				continue;
			}

			switch ( $attribute ) {

				case 'title':
					$schema['properties']['title'] = array(
						'description' => __( 'The title for the object.' ),
						'type'        => 'object',
						'context'     => array( 'view', 'edit', 'embed' ),
						'arg_options' => array(
							'sanitize_callback' => null, // Note: sanitization implemented in self::prepare_item_for_database( )
							'validate_callback' => null, // Note: validation implemented in self::prepare_item_for_database( )
						),
						'properties'  => array(
							'raw' => array(
								'description' => __( 'Title for the object, as it exists in the database.' ),
								'type'        => 'string',
								'context'     => array( 'edit' ),
							),
							'rendered' => array(
								'description' => __( 'HTML title for the object, transformed for display.' ),
								'type'        => 'string',
								'context'     => array( 'view', 'edit', 'embed' ),
								'readonly'    => true,
							),
						),
					);
					break;

				case 'editor':
					$schema['properties']['content'] = array(
						'description' => __( 'The content for the object.' ),
						'type'        => 'object',
						'context'     => array( 'view', 'edit' ),
						'arg_options' => array(
							'sanitize_callback' => null, // Note: sanitization implemented in self::prepare_item_for_database( )
							'validate_callback' => null, // Note: validation implemented in self::prepare_item_for_database( )
						),
						'properties'  => array(
							'raw' => array(
								'description' => __( 'Content for the object, as it exists in the database.' ),
								'type'        => 'string',
								'context'     => array( 'edit' ),
							),
							'rendered' => array(
								'description' => __( 'HTML content for the object, transformed for display.' ),
								'type'        => 'string',
								'context'     => array( 'view', 'edit' ),
								'readonly'    => true,
							),
							'protected'       => array(
								'description' => __( 'Whether the content is protected with a password.' ),
								'type'        => 'boolean',
								'context'     => array( 'view', 'edit', 'embed' ),
								'readonly'    => true,
							),
						),
					);
					break;

				case 'author':
					$schema['properties']['author'] = array(
						'description' => __( 'The ID for the author of the object.' ),
						'type'        => 'integer',
						'context'     => array( 'view', 'edit', 'embed' ),
					);
					break;

				case 'excerpt':
					$schema['properties']['excerpt'] = array(
						'description' => __( 'The excerpt for the object.' ),
						'type'        => 'object',
						'context'     => array( 'view', 'edit', 'embed' ),
						'arg_options' => array(
							'sanitize_callback' => null, // Note: sanitization implemented in self::prepare_item_for_database( )
							'validate_callback' => null, // Note: validation implemented in self::prepare_item_for_database( )
						),
						'properties'  => array(
							'raw' => array(
								'description' => __( 'Excerpt for the object, as it exists in the database.' ),
								'type'        => 'string',
								'context'     => array( 'edit' ),
							),
							'rendered' => array(
								'description' => __( 'HTML excerpt for the object, transformed for display.' ),
								'type'        => 'string',
								'context'     => array( 'view', 'edit', 'embed' ),
								'readonly'    => true,
							),
							'protected'       => array(
								'description' => __( 'Whether the excerpt is protected with a password.' ),
								'type'        => 'boolean',
								'context'     => array( 'view', 'edit', 'embed' ),
								'readonly'    => true,
							),
						),
					);
					break;

				case 'page-attributes':
					$schema['properties']['menu_order'] = array(
						'description' => __( 'The order of the object in relation to other object of its type.' ),
						'type'        => 'integer',
						'context'     => array( 'view', 'edit' ),
					);
					break;

				case 'custom-fields':
					$schema['properties']['meta'] = $this->meta->get_field_schema( );
					break;

			}
		}

		$schema['properties']['sticky'] = array(
			'description' => __( 'Whether or not the object should be treated as sticky.' ),
			'type'        => 'boolean',
			'context'     => array( 'view', 'edit' ),
		);

		$taxonomies = wp_list_filter( get_object_taxonomies( 'post', 'objects' ), array( 'show_in_rest' => true ) );
		foreach ( $taxonomies as $taxonomy ) {
			$base = ! empty( $taxonomy->rest_base ) ? $taxonomy->rest_base : $taxonomy->name;
			$schema['properties'][ $base ] = array(
				/* translators: %s: taxonomy name */
				'description' => sprintf( __( 'The terms assigned to the object in the %s taxonomy.' ), $taxonomy->name ),
				'type'        => 'array',
				'items'       => array(
					'type'    => 'integer',
				),
				'context'     => array( 'view', 'edit' ),
			);
		}

		return $this->add_additional_fields_schema( $schema );
	}

	public function sanitize_post_statuses( $statuses, $request, $parameter ) {
		$statuses = wp_parse_slug_list( $statuses );

		// The default status is different in WP_REST_Attachments_Controller
		$attributes = $request->get_attributes( );
		$default_status = $attributes['args']['status']['default'];

		foreach ( $statuses as $status ) {
			if( $status === $default_status ) {
				continue;
			}

			$post_type_obj = get_post_type_object( 'post' );

			if( current_user_can( $post_type_obj->cap->edit_posts ) ) {
				$result = rest_validate_request_arg( $status, $request, $parameter );
				if( is_wp_error( $result ) ) {
					return $result;
				}
			} else {
				return new WP_Error( 'rest_forbidden_status', __( 'Status is forbidden.' ), array( 'status' => rest_authorization_required_code( ) ) );
			}
		}

		return $statuses;
	}

}